# $Header: /cvsroot/SchemaEditor/XSCardinality.py,v 1.6 2003/09/05 15:16:24 spuelrich Exp $
__version__ = '$Revision: 1.6 $'[11:-2]

class XSCardinality:
    """a counter for cardinalities"""
    def __init__(self, value):
        if isinstance(value, XSCardinality):
            self.value = value.value
            self.unbounded = value.unbounded
        elif value == 'unbounded':
            self.value = 0L
            self.unbounded = 1
        elif value == '':
            # this is a shortcut for the default value
            # of xsd:minOccurs/xsd:maxOccurs
            self.value = 1L
            self.unbounded = 0
        else:
            self.value = long(value)
            self.unbounded = 0
            
    def __str__(self):
        if self.unbounded:
            return "'unbounded'"
        else:
            return str(self.value)
        
    def __repr__(self):
        return 'XSCardinality(%s)' % (self.__str__())

    def __int__(self):
        return int(self.value)

    def __long__(self):
        return self.value
    
    # self + other
    def __add__(self, other):
        if self.unbounded or other.unbounded:
            return XSCardinality('unbounded')
        return XSCardinality(self.value + other.value)

    # self - other
    def __sub__(self, other):
        if self.unbounded or other.unbounded:
            return XSCardinality('unbounded')
        return XSCardinality(self.value - other.value)

    # other + self
    __radd__ = __add__

    # other - self
    def __rsub__(self, other):
        if self.unbounded or other.unbounded:
            return XSCardinality('unbounded')
        return XSCardinality(other.value - self.value)

    # self += other
    def __iadd__(self, other):
        self.unbounded = (self.unbounded or other.unbounded)
        self.value = self.value + other.value
        return self
        
    # self -= other
    def __isub__(self, other):
        self.unbounded = (self.unbounded or other.unbounded)
        self.value = self.value - other.value
        return self
    
    def __neg__(self):
        if self.unbounded:
            return XSCardinality('unbounded')
        return XSCardinality(-self.value)

    def __nonzero__(self):
        return self.unbounded or self.value != 0
    
    def __coerce__(self, other):
        if isinstance(other, XSCardinality):
            return (self, other)
        else:
            return (self, XSCardinality(other))
    
    def __cmp__(self, other):
        if self.unbounded:
            if other.unbounded:
                return 0
            else:
                return 1
        else:
            if other.unbounded:
                return -1
            else:
                return cmp(self.value, other.value)
            
