# $Header: /cvsroot/SchemaEditor/XSCustomTypes.py,v 1.28 2003/09/05 15:16:24 spuelrich Exp $
__version__ = '$Revision: 1.28 $'[11:-2]

import XSExceptions
import XSWebType
import XSType
from xsdTools import xsdURI, seURI, getChildNodesByTagNameNS, getFirstElementChild, getElementValue
from XSFacet import XSFacetRegistry
from XSTypeRegistry import globalXSTypeRegistry
import XSField

from Persistence import Persistent
from AccessControl import ClassSecurityInfo

import re

def extract_documentation(annotation):
    """extracts the TEXT_NODEs from the given xsd:annotation node"""
    result = u''
    for doc in annotation.getElementsByTagNameNS(xsdURI, 'documentation'):
        for node in doc.childNodes:
            result += node.textareaStr()
    result = re.sub('\s+', ' ', result)
    return result

# XXX XSComplexType, XSSimpleType and all the XSType.* should share one interface.
# They do this currently, but this is not enforced by the code...
# add something like class XSTypeInterface and use it as superclass
class XSComplexType(Persistent):
    """A class for the XML Schema complexType"""
    security = ClassSecurityInfo()

    def __init__(self, node):
        self.name = node.getAttribute('name')
        self.nodepath = node.getNodePath('element_id')
        annotation = getFirstElementChild(node)
        if (annotation is not None and
            annotation.localName == 'annotation'):
            self.documentation = extract_documentation(annotation)
        else:
            self.documentation = ''

    def __str__(self):
        return '<XSComplexType name="%s" nodepath="%s">' % (self.name, self.nodepath)
    
    security.declarePublic('render')
    def render(self, context, descend=1):
        """renders a complexType.
        if descend==0 it will not descend the XSI tree.
        This should be used only for complexType-extensions."""
        request = context.REQUEST
        xsd_node = request.node
        xsi_node = request.xsi_node
        schema_editor = context.schema_editor
        service_editor = context.service_editor

        if descend:
            # descend one level within the XSI
            previous_parent = request.xsi_parent_node
            request.set('xsi_parent_node', xsi_node)
            request.set('xsi_node', getFirstElementChild(xsi_node))
        if schema_editor.is_edit_node(xsi_node):
            service_editor.setDocumentEditor(xsd_node, 'service_schema_edit')

        defining_node = xsd_node.ownerDocument.resolveNodePath(self.nodepath)
        # call the widget for the node, which defines the type.
        # we have to call the render_real method, because render() does nothing.
        result = service_editor.getWidget(defining_node).render_real()
        if descend:
            # go up again
            request.set('xsi_node', xsi_node)
            request.set('xsi_parent_node', previous_parent)
        if schema_editor.is_edit_node(xsi_node):
            service_editor.setDocumentEditor(xsd_node, 'service_schema_normal')
        return result

    security.declarePublic('create_instance')
    def create_instance(self, context, value=None):
        xsd = context.REQUEST.node.ownerDocument
        defining_node = xsd.resolveNodePath(self.nodepath)
        # derived complexTypes are handeled within the widgets
        widget = context.service_editor.getWidget(defining_node)
        return widget.render()
    
    security.declarePublic('render_attributes')
    def render_attributes(self, context):
        xsd = context.REQUEST.node.ownerDocument
        xsi_node = context.REQUEST.xsi_node
        defining_node = xsd.resolveNodePath(self.nodepath)
        attribute_declarations = getChildNodesByTagNameNS(defining_node, 'attribute')
        attrs = []
        for attribute in attribute_declarations:
            # 'attribute' contains only the attribute declaration
            # <xsd:attribute name="" type="" use=""/>
            name = attribute.getAttribute('name')
            if xsi_node.hasAttribute(name):
                attrs.append('%s="%s"' % (name, xsi_node.getAttribute(name)))
        if attrs:
            attrs.insert(0, '')
        return ' '.join(attrs)

    # look at create_instance!
    # exactly the same thing a save method would do.
    save = create_instance

    security.declarePublic('has_simple_content')
    def has_simple_content(self):
        return 0

    security.declarePublic('get_info')
    def get_info(self):
        return self.documentation

def XSSimpleType(node):
    basetype = None
    derivation_node = None
    derivation_method = None
    documentation = ''
    name = node.getAttribute('name')
    nodepath = node.getNodePath('element_id')
    # access the context via the node. acquisition...
    schema_editor = node.schema_editor
    for child in node.childNodes:
        if child.namespaceURI != xsdURI:
            # ignore non-XS elements
            continue
        if child.localName in ('restriction', 'list'):
            derivation_node = child
            derivation_method = child.localName
        elif child.localName == 'annotation':
            documentation = extract_documentation(child)
        elif child.localName == 'union':
            raise XSExceptions.XSNotSupported('derivation sort "union" is not yet supported')
        else:
            raise XSExceptions.XSSchemaError('invalid content of <xsd:simpleType>: %s' % child.localName)
    if derivation_method is None:
            raise XSExceptions.XSSchemaError('no element for derivation found')
    # search the base type
    if derivation_method == 'restriction' and derivation_node.hasAttribute('base'):
        basename = derivation_node.getAttribute('base')
        basetype = schema_editor.get_type_by_name(basename)
        if basetype is None:
            # XXX could investigate, whether a simpleType child defines this type... (ugly Schema!)
            raise XSExceptions.XSUnknownTypeException, "unknown type %s" % (basename)
    elif derivation_method == 'list' and derivation_node.hasAttribute('itemType'):
        basename = derivation_node.getAttribute('itemType')
        basetype = schema_editor.get_type_by_name(basename)
    elif derivation_method in ('restriction', 'list'):
        # search and register recursivley the underlying xsd:simpleTypes
        nl = getChildNodesByTagNameNS(derivation_node, 'simpleType', xsdURI)
        if not nl:
            raise XSSchemaError('no basetype given for simpleType: name="%s" nodepath="%s"' %
                                (name, nodepath))
        basetype_definition = nl[0]
        basetype = XSSimpleType(basetype_definition)
        schema_editor.register_type(basetype)
    else:
        # dunno. something went wrong
        pass
    # create new type
    if derivation_method == 'restriction':
        newtype = XSCustomSimpleTypeRestriction(basetype, name, nodepath, documentation)
    elif derivation_method == 'list':
        newtype = XSCustomSimpleTypeList(basetype, name, nodepath, documentation)
    elif derivation_method == 'union':
        #newtype = XSCustomSimpleTypeUnion(basetype, name, nodepath)
        raise XSExceptions.XSNotSupported('derivation sort "union" is not supported')
    # is default_value set
    if node.hasAttributeNS(seURI, 'defaultValue'):
        value = newtype.get_nofacet_value()
        value.set_value(node.getAttributeNS(seURI, 'defaultValue'))
        newtype.default_value = value
    # search for facets within derivation_node
    for child in derivation_node.childNodes:
        if (child.namespaceURI != xsdURI or child.localName in ('annotation', 'simpleType')):
            # ignore annotations. simpleTypes already handeled
            continue
        # XXX really decode from unicode?? XML Schema tag names are all ascii...
        facet_name = child.localName
        facet_value = child.getAttribute('value')
        newtype.add_facet_value(facet_name, facet_value)
        
    return newtype


class XSCustomSimpleType(Persistent):
    name = None
    nodepath = None
    default_value = None
    documentation = ''
    facet_class = None
    nofacet_class = None
    allowed_facets = None
    value = None
    normalized_value = None
    original_value = None
    sort = None
    basetype_name = None
    basetype = None

    def add_facet_value(self, facet_name, facet_value):
        """add facet with facet_name and facet_value"""
        if facet_name not in self.allowed_facets:
            raise XSExceptions.XSFacetNotAllowed('facet "%s" not allowed for %s' %
                                                 (facet_name, self.basetype_name))
        facets = self.facets
        # check if the facet already exists
        if facets.has_key(facet_name):
            facet = facets[facet_name]
        else:
            facet = XSFacetRegistry.get_facet(facet_name)
            facets[facet_name] = facet
        if facet.value_type is not None:
            # this facets requires a special type
            # try to get the class of this type from the XSType module
            typeklass = getattr(XSType, 'XSType_%s' % facet.value_type, None)
            if typeklass is None:
                raise XSUnknownTypeException('no XSType.XSType_%s available' % facet.value_type)
            facet_value = typeklass(facet_value)
        else:
            # the facet requires the same type as the validated value
            # don't use the XSWebType. XSType... is enough (includes facets!)
            # this part validates the value given in the schema.
            facet_value = self.get_basetype().facet_class(facet_value)

        facet.add(facet_value.get_nofacet_value())
        self.facets = facets
        return

    def get_derivation_hierarchy(self):
        if self.basetype is None:
            # XS standard built-in types
            suffix = self.basetype_name
        else:
            suffix = self.get_basetype().get_derivation_hierarchy()
        return ', '.join((self.name, suffix))

    def get_basetype(self):
        if self.basetype is not None:
            return self.basetype
        return globalXSTypeRegistry.get_type(name=self.basetype_name)

    def __repr__(self):
        return '%s([%s], "%s", %s)' % (self.__class__.__name__,
                                       self.get_derivation_hierarchy(),
                                       self.unicode(),
                                       self.facets.keys())

class XSCustomSimpleTypeList(XSCustomSimpleType):
    security = ClassSecurityInfo()
    def __init__(self, basetype, newname, nodepath, documentation):
        self.name = newname
        self.nodepath = nodepath
        self.default_value = None
        self.documentation = documentation
        self.facets = {}
        self.facet_class = basetype.facet_class
        self.nofacet_class = basetype.nofacet_class
        self.allowed_facets = ('enumeration', 'pattern',
                               'length', 'minLength', 'maxLength')
        if hasattr(basetype, 'basetype_name'):
            self.basetype_name = basetype.basetype_name
        else:
            self.basetype_name = basetype.name
        # do not save references to built-in standard types
        if issubclass(basetype.__class__, XSWebType.XSWebType):
            self.basetype = None
        else:
            self.basetype = basetype
        self.value = None # this will be a tuple! a tuple of subvalues
        self.normalized_value = None
        self.original_value = None
        self.sort = 'list'
        self.field = XSField.XSField_string(XSField.XSField_string.meta_type, whitespace_preserve="1", unicode="1")

    split = re.compile('\s+').split

    # interface from XSWebType
    security.declarePublic('render')
    def render(self, context):
        xsd_node = context.REQUEST.node
        return context.service_editor.getWidget(xsd_node).render_real()

    security.declarePublic('render_field')
    def render_field(self, xsi_node, value=None):
        if value is None:
            value = getElementValue(xsi_node)
        elif isinstance(value, type(())) or isinstance(value, type([])):
            value = ' '.join(value)
        return self.field.render(xsi_node, value)

    security.declarePublic('create_instance')
    def create_instance(self, context, value=None):
        how_often = 1
        facets = self.facets
        if facets.has_key('minLength'):
            how_often = long(facets['minLength'].values[-1])
        elif facets.has_key('length'):
            how_often = long(facets['length'].values[-1])
        xsi_parent = context.REQUEST.xsi_parent_node
        if value is None:
            # compute default value as a XSType_nofacet
            value = self.get_default_value()
        content = ' '.join([value.unicode(),] * how_often)
        node = xsi_parent.ownerDocument.createTextNode(content)
        xsi_parent.appendChild(node)
        return 'created %s instance' % (self.name)

        if value is None:
            value = self.get_default_value()
        return self.get_basetype().create_instance(context, value)

    security.declarePublic('validate_from_request')
    def validate_from_request(self, xsi_node, REQUEST):
        # this will NOT set the value!
        # circumvent my own validate_from_request, if there is a xsd:enumeration facet
        return self.field.validate(xsi_node, REQUEST)
        #return self.get_basetype().validate_from_request(xsi_node, REQUEST)

    security.declarePublic('unicode')
    def unicode(self):
        if hasattr(self.get_basetype(), 'use_original_value'):
            if isinstance(self.original_value, type(u'')):
                return self.original_value
            elif isinstance(self.original_value, type('')):
                return unicode(self.original_value, 'UTF-8')
            elif isinstance(self.normalized_value, type(u'')):
                return ' '.join(self.normalized_value)
            elif isinstance(self.normalized_value, type('')):
                return unicode(' '.join(self.normalized_value), 'UTF-8')
        return unicode(str(self), 'UTF-8')

    security.declarePublic('has_simple_content')
    def has_simple_content(self):
        return 1

    security.declarePublic('get_info')
    def get_info(self):
        if self.documentation:
            return self.documentation
        else:
            return "derived from: %s" % self.get_basetype().get_info()

    # interface from XSTypeFacet
    security.declarePublic('set_value')
    def set_value(self, value=None):
        self.original_value = value
        self.normalized_value = self.normalize_value(value)
        self.value = self._set_value_helper(self.normalized_value)
        self.validate(self.value, self.normalized_value)
        
    def normalize_value(self, value):
        return tuple(self.split(value))
    
    security.declarePublic('get_nofacet_value')
    def get_nofacet_value(self, value=None):
        return self.get_basetype().get_nofacet_value(value)
    
    def validate(self, value, normalized_value):
        # validate against my own facets
        for facet in self.facets.values():
            if facet.name in ('xsd:enumeration', 'xsd:pattern'):
                for (subvalue, subnormvalue) in map(None, value, normalized_value):
                    facet.validate(subvalue, subnormvalue)
            else:
                facet.validate(value, normalized_value)
        # validate against basetypes facets
        basetype = self.get_basetype()
        for (subvalue, subnormvalue) in map(None, value, normalized_value):
            basetype.validate(subvalue, subnormvalue)

    # interface from XSType_nofacet
    def get_value(self):
        return self.value

    def get_default_value(self, plain=0):
        """nearly a plain copy of XSType.XSType.get_default_value()"""
        value = self.default_value
        if value is None:
            value = self._get_default_value_facet()
            # value is a XSType_nofacet or None
            if value is not None:
                if plain:
                    value = value.get_value()
            else: # value is None
                value = self._get_default_value_nofacet()
                # now value is a plain type
                if not plain:
                    value = self.nofacet_class(value)
        return value

    def _get_default_value_facet(self):
        facets = self.facets
        value = None
        if facets.has_key('enumeration'):
            value = facets['enumeration'].values[-1]
        else:
            value = self.get_basetype()._get_default_value_facet()
        return value

    def _get_default_value_nofacet(self):
        return self.get_basetype()._get_default_value_nofacet()

    def _set_value_helper(self, value):
        result = []
        for subvalue in value:
            result.append(self.get_basetype().get_nofacet_value(subvalue))
        return tuple(result)
        #return self.get_basetype()._set_value_helper(value)

    # no coerce, no compare!
    ## def __coerce__(self, other):
    ##     return coerce(self.get_basetype(), other)
    ## 
    ## def __cmp__(self, other):
    ##     return cmp(self.value, other.value)

    def __str__(self):
        if self.value is None:
            return u'<not yet set>'
        return u' '.join(map(str, self.value))


class XSCustomSimpleTypeRestriction(XSCustomSimpleType):
    security = ClassSecurityInfo()
    def __init__(self, basetype, newname, nodepath, documentation):
        self.name = newname
        self.nodepath = nodepath
        self.default_value = None
        self.documentation = documentation
        self.facets = {}
        self.facet_class = basetype.facet_class
        self.nofacet_class = basetype.nofacet_class
        self.allowed_facets = basetype.allowed_facets
        if hasattr(basetype, 'basetype_name'):
            self.basetype_name = basetype.basetype_name
        else:
            self.basetype_name = basetype.name
        # do not save references to built-in standard types
        if issubclass(basetype.__class__, XSWebType.XSWebType):
            self.basetype = None
        else:
            self.basetype = basetype
        self.value = None
        self.normalized_value = None
        self.original_value = None
        self.sort = 'atomic'

    # interface from XSWebType
    security.declarePublic('render')
    def render(self, context):
        xsi_node = context.REQUEST.xsi_node
        # XXX very ugly.
        if context.schema_editor.is_edit_node(xsi_node, check_ancestors=1):
            return self.render_field(xsi_node)
        return self.get_basetype().render(context)

    security.declarePublic('render_field')
    def render_field(self, xsi_node, value=None):
        if self.facets.has_key('enumeration') and self.get_basetype().sort == 'atomic':
            return self.facets['enumeration'].render_field(xsi_node)
        return self.get_basetype().render_field(xsi_node, value)

    security.declarePublic('create_instance')
    def create_instance(self, context, value=None):
        if value is None:
            value = self.get_default_value()
        return self.get_basetype().create_instance(context, value)

    security.declarePublic('validate_from_request')
    def validate_from_request(self, xsi_node, REQUEST):
        # this will NOT set the value!
        # circumvent my own validate_from_request, if there is a xsd:enumeration facet
        if self.facets.has_key('enumeration'):
            return self.facets['enumeration'].validate_field(xsi_node, REQUEST)
        else:
            return self.get_basetype().validate_from_request(xsi_node, REQUEST)

    security.declarePublic('unicode')
    def unicode(self):
        if hasattr(self.get_basetype(), 'use_original_value'):
            if isinstance(self.original_value, type(u'')):
                return self.original_value
            elif isinstance(self.original_value, type('')):
                return unicode(self.original_value, 'UTF-8')
            elif isinstance(self.normalized_value, type(u'')):
                return self.normalized_value
            elif isinstance(self.normalized_value, type('')):
                return unicode(self.normalized_value, 'UTF-8')
        return unicode(str(self), 'UTF-8')

    security.declarePublic('has_simple_content')
    def has_simple_content(self):
        return 1

    security.declarePublic('get_info')
    def get_info(self):
        if self.documentation:
            return self.documentation
        else:
            return "derived from: %s" % self.get_basetype().get_info()

    # interface from XSTypeFacet
    security.declarePublic('set_value')
    def set_value(self, value=None):
        self.original_value = value
        self.normalized_value = self.normalize_value(value)
        self.value = self._set_value_helper(self.normalized_value)
        self.get_basetype().validate(self.value, self.normalized_value)
        self.validate(self.value, self.normalized_value)
        
    def normalize_value(self, value):
        return self.get_basetype().normalize_value(value)
    
    security.declarePublic('get_nofacet_value')
    def get_nofacet_value(self, value=None):
        return self.get_basetype().get_nofacet_value(value)
    
    def validate(self, value, normalized_value):
        for facet in self.facets.values():
            if facet.name != 'xsd:whiteSpace':
                facet.validate(value, normalized_value)

    # interface from XSType_nofacet
    def get_value(self):
        return self.value

    def get_default_value(self, plain=0):
        """nearly a plain copy of XSType.XSType.get_default_value()"""
        value = self.default_value
        if value is None:
            value = self._get_default_value_facet()
            # value is a XSType_nofacet or None
            if value is not None:
                if plain:
                    value = value.get_value()
            else: # value is None
                value = self._get_default_value_nofacet()
                # now value is a plain type
                if not plain:
                    value = self.nofacet_class(value)
        return value

    def _get_default_value_facet(self):
        facets = self.facets
        value = None
        if facets.has_key('enumeration'):
            value = facets['enumeration'].values[-1]
            
        elif facets.has_key('maxInclusive'):
            value = facets['maxInclusive'].values[-1]

        elif facets.has_key('minInclusive'):
            value = facets['minInclusive'].values[-1]

        else:
            value = self.get_basetype()._get_default_value_facet()
        return value

    def _get_default_value_nofacet(self):
        return self.get_basetype()._get_default_value_nofacet()

    def _set_value_helper(self, value):
        return self.get_basetype()._set_value_helper(value)
    
    def __coerce__(self, other):
        return coerce(self.get_basetype(), other)

    def __cmp__(self, other):
        return cmp(self.value, other.value)

    def __str__(self):
        return str(self.get_nofacet_value(self.value))

