# $Header: /cvsroot/SchemaEditor/XSPrettyPrinter.py,v 1.13 2003/08/25 08:25:36 spuelrich Exp $
"""
Gibt einen Teil eines XML Schema Dokuments in einer hbschen Textdarstellung zurck.
Der XML Baum wird nur bis zu einer vorgegebenen Tiefe durchlaufen.
Es werden nur ausgewhlte Elemente und Attribute dargestellt.
"""
from xsdTools import xsdURI, seURI

structure = { 'schema':('element', 'annotation', 'complexType', 'simpleType'),
              'element':('complexType', 'simpleType', 'annotation'),
              'complexType':('complexContent', 'sequence', 'choice', 'all', 'annotation'),
              'complexContent':('extension',),
              'extension':('sequence', 'choice', 'all', 'annotation'),
              'sequence':('element', 'sequence', 'choice', 'annotation'),
              'choice':('element', 'sequence', 'choice', 'annotation'),
              'all':('element', 'annotation',),
              'simpleType':('restriction', 'list', 'annotation'),
              'restriction':('pattern', 'enumeration', 'whiteSpace',
                             'maxInclusive', 'maxExclusive', 'minInclusive', 'minExclusive',
                             'length', 'minLength', 'maxLength',
                             'totalDigits', 'fractionDigits','annotation'),
              'list':(),
              'annotation':('documentation', 'appinfo'),
              'documentation':(),
              'appinfo':(),
              'pattern':(),
              'enumeration':(),
              'whiteSpace':(),
              'maxInclusive':(),
              'maxExclusive':(),
              'minInclusive':(),
              'minExclusive':(),
              'length':(),
              'minLength':(),
              'maxLength':(),
              'totalDigits':(),
              'fractionDigits':()
              }

attributes = { 'schema':(),
               'element':('minOccurs', 'maxOccurs', 'type', 'name'),
               'complexType':('name',),
               'complexContent':(),
               'extension':('base',),
               'sequence':('minOccurs', 'maxOccurs'),
               'choice':('minOccurs', 'maxOccurs'),
               'all':('minOccurs', 'maxOccurs'),
               'simpleType':('name',),
               'restriction':('base',),
               'list':('itemType',),
               'annotation':(),
               'documentation':(),
               'appinfo':(),
               'pattern':('value',),
               'enumeration':('value',),
               'whiteSpace':('value',),
               'maxInclusive':('value',),
               'maxExclusive':('value',),
               'minInclusive':('value',),
               'minExclusive':('value',),
               'length':('value',),
               'minLength':('value',),
               'maxLength':('value',),
               'totalDigits':('value',),
               'fractionDigits':('value',)
              }

class XSPrettyPrinter:

    def __init__(self, indent='  '):
        self.indent = indent
    
    def pprint(self, node, maxDepth=-1, maxLines=-1):
        if node is None:
            return 'None'
        if node.namespaceURI == xsdURI:
            result = self._pprint_xmlschema(node, maxDepth, 0)
        else:
            result = self._pprint_xml(node, maxDepth, 0)
        if maxLines == -1:
            return result
        # split result and remove empty lines
        lines = [line for line in result.splitlines() if not line.isspace()]
        if len(lines) < maxLines:
            # result is OK
            return '\n'.join(lines)
        # take maxLines/2 from the top and from the bottom of the result
        new_lines = lines[:(maxLines/2)] + ['...'] + lines[(-maxLines/2):]
        return "\n".join(new_lines)
    
    def _pprint_xmlschema(self, node, maxDepth, depth):
        if (node.nodeType != node.ELEMENT_NODE or
            node.namespaceURI != xsdURI):
            return ''
        if( depth == maxDepth ):
            return '%s...\n' % (self.indent * depth)
        
        lname = node.localName

        # attribute
        attrs = []
        for att in node.attributes.values():
            if att.name in attributes[lname] or att.namespaceURI == seURI:
                attrs.append('%s="%s"' % (att.name, att.value))
        if attrs:
            attrs = ' ' + ' '.join(attrs)
        else:
            attrs = ''
        # inhalt
        if lname in ('documentation', 'appinfo'):
            # text
            content = ''.join([tn.nodeValue for tn in node.childNodes if tn.nodeType == node.TEXT_NODE])
            content = self._pprint_text(content, depth+1)
        else:
            # kindknoten
            content = ''
            for child in node.childNodes:
                if child.localName not in structure[lname]:
                    continue
                content += self._pprint_xmlschema(child, maxDepth, depth+1)
        # zusammenbauen
        tagName = node.tagName
        if content:
            result = ('%s<%s%s>\n' % (self.indent * depth, tagName, attrs) +
                      content +
                      '%s</%s>\n' % (self.indent * depth, tagName))
        else:
            result = '%s<%s%s/>\n' % (self.indent * depth, tagName, attrs)
        return result

    def _pprint_xml(self, node, maxDepth, depth):
        """
        gibt ein XML Dokument bis zu einer vorgegebenen Tiefe aus.
        attribute werden ignoriert
        """
        if depth == maxDepth:
            return '%s...\n' % (self.indent * depth)
        
        if node.nodeType == node.ELEMENT_NODE:
            name = node.nodeName
            # kindknoten
            content = ''
            for child in node.childNodes:
                content += self._pprint_xml(child, maxDepth, depth+1)
                # zusammenbauen
            if content:
                return ('%s<%s>\n' % (self.indent * depth, name,) +
                        content +
                        '%s</%s>\n' % (self.indent * depth, name))
            else:
                return '%s<%s/>\n' % (self.indent * depth, name)
        elif node.nodeType == node.TEXT_NODE:
            text = node.nodeValue
            return self._pprint_text(text, depth)
        else:
            return ''

    def _pprint_text(self, text, depth):
        lines = text.splitlines()
        results = []
        for line in lines:
            if not line.isspace():
                line = line.strip()
                line = "%s%s\n" % (self.indent * depth, line)
                results.append(line)
        return ''.join(results)
TheXSPrettyPrinter = XSPrettyPrinter()
