# $Header: /cvsroot/SchemaEditor/XSType.py,v 1.34 2003/08/18 13:08:45 spuelrich Exp $
from DateTime import DateTime

import XSField
import XSType_nofacet
import XSFacet
from XSExceptions import XSNotSupported

# XSTypeFacet contains the xml schema facets support for the XSType_nofacet types
class XSTypeFacet:
    def set_value(self, value=None):
        if type(value) == type(''):
            value = unicode(value, 'UTF-8')
        elif type(value) == type(u''):
            pass
        elif value == None:
            pass
        else:
            value = unicode(str(value))
        self.original_value = value
        self.normalized_value = self.normalize_value(value)
        self.value = self._set_value_helper(self.normalized_value)
        self.validate(self.value, self.normalized_value)

    def normalize_value(self, value):
        if self.facets.has_key('whiteSpace') and type(value) in (type(''), type(u'')):
            return self.facets['whiteSpace'].validate(value, None)
        else:
            return value

    def unicode(self):
        if hasattr(self, 'use_original_value'):
            if isinstance(self.original_value, type(u'')):
                return self.original_value
            elif isinstance(self.original_value, type('')):
                return unicode(self.original_value, 'UTF-8')
            elif isinstance(self.normalized_value, type(u'')):
                return self.normalized_value
            elif isinstance(self.normalized_value, type('')):
                return unicode(self.normalized_value, 'UTF-8')
        return unicode(str(self), 'UTF-8')
        
    def get_nofacet_value(self, value=None):
        # XXX test this
        if value is None:
            return self.nofacet_class(self.value)
        return self.nofacet_class(value)
    
    def validate(self, value, normalized_value):
        # value and normalized_value have to be given explicitly
        # make it possible to validate any value against this type
        for facet in self.facets.values():
            if facet.name != 'xsd:whiteSpace':
                facet.validate(value, normalized_value)

    def get_default_value(self, plain=0):
        """tries to compute a default value from the facets.
        otherwise the nofacet default value is returned.
        if plain is set to 1, this returns a plain Python type,
        instead of a XSType_nofacet type."""
        value = self._get_default_value_facet()
        # value is a XSType_nofacet or None
        if value is not None:
            if plain:
                value = value.get_value()
        else: # value is None
            value = self._get_default_value_nofacet()
            # now value is a plain type
            if not plain:
                value = self.nofacet_class(value)
        return value

    def _get_default_value_facet(self):
        """tries to compute a defaultValue from the facets,
        which belong to this type."""
        facets = self.facets
        value = None
        if facets.has_key('enumeration'):
            value = facets['enumeration'].values[-1]
            
        elif facets.has_key('maxInclusive'):
            value = facets['maxInclusive'].values[-1]

        elif facets.has_key('minInclusive'):
            value = facets['minInclusive'].values[-1]
        return value

class XSType_int(XSTypeFacet, XSType_nofacet.XSType_nofacet_int):
    facets = {}
    name = 'xsd:int'
    nofacet_class = XSType_nofacet.XSType_nofacet_int
    allowed_facets = ('pattern', 'enumeration', 'whiteSpace',
                      'maxInclusive', 'maxExclusive', 'minInclusive', 'minExclusive',
                      'totalDigits', 'fractionDigits')

class XSType_short(XSTypeFacet, XSType_nofacet.XSType_nofacet_short):
    facets = {
        'maxInclusive':XSFacet.XSFacet_maxInclusive((XSType_nofacet.XSType_nofacet_short(32767),)),
        'minInclusive':XSFacet.XSFacet_minInclusive((XSType_nofacet.XSType_nofacet_short(-32768),))
        }
    name = 'xsd:short'
    nofacet_class = XSType_nofacet.XSType_nofacet_short
    allowed_facets = ('pattern', 'enumeration', 'whiteSpace',
                      'maxInclusive', 'maxExclusive', 'minInclusive', 'minExclusive',
                      'totalDigits', 'fractionDigits')

class XSType_date(XSTypeFacet, XSType_nofacet.XSType_nofacet_date):
    facets = {}
    nofacet_class = XSType_nofacet.XSType_nofacet_date
    name = 'xsd:date'
    allowed_facets = ('pattern', 'enumeration', 'whiteSpace',
                      'maxInclusive', 'maxExclusive', 'minInclusive', 'minExclusive')
    
class XSType_dateTime(XSTypeFacet, XSType_nofacet.XSType_nofacet_dateTime):
    facets = {}
    nofacet_class = XSType_nofacet.XSType_nofacet_dateTime
    name = 'xsd:dateTime'
    allowed_facets = ('pattern', 'enumeration', 'whiteSpace',
                      'maxInclusive', 'maxExclusive', 'minInclusive', 'minExclusive')

class XSType_string(XSTypeFacet, XSType_nofacet.XSType_nofacet_string):
    facets = {}
    use_original_value = 1
    nofacet_class = XSType_nofacet.XSType_nofacet_string
    name = 'xsd:string'
    allowed_facets = ('length', 'minLength', 'maxLength',
                      'pattern', 'enumeration', 'whiteSpace')

class XSType_float(XSTypeFacet, XSType_nofacet.XSType_nofacet_float):
    facets = {}
    nofacet_class = XSType_nofacet.XSType_nofacet_float
    name = 'xsd:float'
    allowed_facets = ('pattern', 'enumeration', 'whiteSpace',
                      'maxInclusive', 'maxExclusive', 'minInclusive', 'minExclusive')

class XSType_double(XSTypeFacet, XSType_nofacet.XSType_nofacet_double):
    facets = {}
    nofacet_class = XSType_nofacet.XSType_nofacet_double
    name = 'xsd:double'
    allowed_facets = ('pattern', 'enumeration', 'whiteSpace',
                      'maxInclusive', 'maxExclusive', 'minInclusive', 'minExclusive')

class XSType_boolean(XSTypeFacet, XSType_nofacet.XSType_nofacet_boolean):
    facets = {}
    nofacet_class = XSType_nofacet.XSType_nofacet_boolean
    name = 'xsd:boolean'
    allowed_facets = ('pattern', 'whiteSpace')

class XSType_integer(XSTypeFacet, XSType_nofacet.XSType_nofacet_integer):
    facets = {}
    nofacet_class = XSType_nofacet.XSType_nofacet_integer
    name = 'xsd:integer'
    allowed_facets = ('pattern', 'enumeration', 'whiteSpace',
                      'maxInclusive', 'maxExclusive', 'minInclusive', 'minExclusive',
                      'totalDigits', 'fractionDigits')

class XSType_positiveInteger(XSTypeFacet, XSType_nofacet.XSType_nofacet_positiveInteger):
    facets = {
        'minInclusive':XSFacet.XSFacet_minInclusive((XSType_nofacet.XSType_nofacet_positiveInteger(1),))
        }
    nofacet_class = XSType_nofacet.XSType_nofacet_positiveInteger
    name = 'xsd:positiveInteger'
    allowed_facets = ('pattern', 'enumeration', 'whiteSpace',
                      'maxInclusive', 'maxExclusive', 'minInclusive', 'minExclusive',
                      'totalDigits', 'fractionDigits')

class XSType_nonNegativeInteger(XSTypeFacet, XSType_nofacet.XSType_nofacet_nonNegativeInteger):
    facets = {
        'minInclusive':XSFacet.XSFacet_minInclusive((XSType_nofacet.XSType_nofacet_nonNegativeInteger(0),))
        }
    nofacet_class = XSType_nofacet.XSType_nofacet_nonNegativeInteger
    name = 'xsd:nonNegativeInteger'
    allowed_facets = ('pattern', 'enumeration', 'whiteSpace',
                      'maxInclusive', 'maxExclusive', 'minInclusive', 'minExclusive',
                      'totalDigits', 'fractionDigits')

class XSType_nonPositiveInteger(XSTypeFacet, XSType_nofacet.XSType_nofacet_nonPositiveInteger):
    facets = {
        'maxInclusive':XSFacet.XSFacet_maxInclusive((XSType_nofacet.XSType_nofacet_nonPositiveInteger(0),))
        }
    nofacet_class = XSType_nofacet.XSType_nofacet_nonPositiveInteger
    name = 'xsd:nonPositiveInteger'
    allowed_facets = ('pattern', 'enumeration', 'whiteSpace',
                      'maxInclusive', 'maxExclusive', 'minInclusive', 'minExclusive',
                      'totalDigits', 'fractionDigits')

class XSType_negativeInteger(XSTypeFacet, XSType_nofacet.XSType_nofacet_negativeInteger):
    facets = {
        'maxInclusive':XSFacet.XSFacet_maxInclusive((XSType_nofacet.XSType_nofacet_negativeInteger(-1),))
        }
    nofacet_class = XSType_nofacet.XSType_nofacet_negativeInteger
    name = 'xsd:negativeInteger'
    allowed_facets = ('pattern', 'enumeration', 'whiteSpace',
                      'maxInclusive', 'maxExclusive', 'minInclusive', 'minExclusive',
                      'totalDigits', 'fractionDigits')

class XSType_long(XSTypeFacet, XSType_nofacet.XSType_nofacet_long):
    facets = {
        'maxInclusive':XSFacet.XSFacet_maxInclusive((XSType_nofacet.XSType_nofacet_long(9223372036854775807L),)),
        'minInclusive':XSFacet.XSFacet_minInclusive((XSType_nofacet.XSType_nofacet_long(-9223372036854775808L),))
        }
    name = 'xsd:long'
    nofacet_class = XSType_nofacet.XSType_nofacet_long
    allowed_facets = ('pattern', 'enumeration', 'whiteSpace',
                      'maxInclusive', 'maxExclusive', 'minInclusive', 'minExclusive',
                      'totalDigits', 'fractionDigits')

class XSType_byte(XSTypeFacet, XSType_nofacet.XSType_nofacet_byte):
    facets = {
        'maxInclusive':XSFacet.XSFacet_maxInclusive((XSType_nofacet.XSType_nofacet_byte(127),)),
        'minInclusive':XSFacet.XSFacet_minInclusive((XSType_nofacet.XSType_nofacet_byte(-128),))
        }
    name = 'xsd:byte'
    nofacet_class = XSType_nofacet.XSType_nofacet_byte
    allowed_facets = ('pattern', 'enumeration', 'whiteSpace',
                      'maxInclusive', 'maxExclusive', 'minInclusive', 'minExclusive',
                      'totalDigits', 'fractionDigits')

class XSType_unsignedLong(XSTypeFacet, XSType_nofacet.XSType_nofacet_unsignedLong):
    facets = {
        'maxInclusive':XSFacet.XSFacet_maxInclusive((XSType_nofacet.XSType_nofacet_unsignedLong(18446744073709551615L),)),
        'minInclusive':XSFacet.XSFacet_minInclusive((XSType_nofacet.XSType_nofacet_unsignedLong(0),))
        }
    name = 'xsd:unsignedLong'
    nofacet_class = XSType_nofacet.XSType_nofacet_unsignedLong
    allowed_facets = ('pattern', 'enumeration', 'whiteSpace',
                      'maxInclusive', 'maxExclusive', 'minInclusive', 'minExclusive',
                      'totalDigits', 'fractionDigits')

class XSType_unsignedInt(XSTypeFacet, XSType_nofacet.XSType_nofacet_unsignedInt):
    facets = {
        'maxInclusive':XSFacet.XSFacet_maxInclusive((XSType_nofacet.XSType_nofacet_unsignedInt(4294967295L),)),
        'minInclusive':XSFacet.XSFacet_minInclusive((XSType_nofacet.XSType_nofacet_unsignedInt(0),))
        }
    name = 'xsd:unsignedInt'
    nofacet_class = XSType_nofacet.XSType_nofacet_unsignedInt
    allowed_facets = ('pattern', 'enumeration', 'whiteSpace',
                      'maxInclusive', 'maxExclusive', 'minInclusive', 'minExclusive',
                      'totalDigits', 'fractionDigits')

class XSType_unsignedShort(XSTypeFacet, XSType_nofacet.XSType_nofacet_unsignedShort):
    facets = {
        'maxInclusive':XSFacet.XSFacet_maxInclusive((XSType_nofacet.XSType_nofacet_unsignedShort(65535),)),
        'minInclusive':XSFacet.XSFacet_minInclusive((XSType_nofacet.XSType_nofacet_unsignedShort(0),))
        }
    name = 'xsd:unsignedShort'
    nofacet_class = XSType_nofacet.XSType_nofacet_unsignedShort
    allowed_facets = ('pattern', 'enumeration', 'whiteSpace',
                      'maxInclusive', 'maxExclusive', 'minInclusive', 'minExclusive',
                      'totalDigits', 'fractionDigits')

class XSType_unsignedByte(XSTypeFacet, XSType_nofacet.XSType_nofacet_unsignedByte):
    facets = {
        'maxInclusive':XSFacet.XSFacet_maxInclusive((XSType_nofacet.XSType_nofacet_unsignedByte(255),)),
        'minInclusive':XSFacet.XSFacet_minInclusive((XSType_nofacet.XSType_nofacet_unsignedByte(0),))
        }
    name = 'xsd:unsignedByte'
    nofacet_class = XSType_nofacet.XSType_nofacet_unsignedByte
    allowed_facets = ('pattern', 'enumeration', 'whiteSpace',
                      'maxInclusive', 'maxExclusive', 'minInclusive', 'minExclusive',
                      'totalDigits', 'fractionDigits')
