# $Header: /cvsroot/SchemaEditor/XSType_nofacet.py,v 1.15 2003/09/05 15:16:24 spuelrich Exp $
__version__ = '$Revision: 1.15 $'[11:-2]

import DateTime
from XSExceptions import XSValidationError

# base class
class XSType_nofacet: #(Persistent):
    sort = 'atomic'
    def __init__(self, value=None):
        self.set_value(value)
        
    def set_value(self, value=None):
        if type(value) not in (type(''), type(u''), type(None), type(1.0)):
            value = str(value)
        self.value = self._set_value_helper(value)

    def _set_value_helper(self, value):
        pass

    def get_value(self):
        """
        returns the value in a form suitable for Formulator fields.
        """
        return self.value

    def get_default_value(self, plain=0):
        """computes the default value of the type"""
        # _get_default_value_nofacet() is implemented by the several sub classes
        return self._get_default_value_nofacet()

    def _get_default_value_nofacet(self):
        pass
    
    def __cmp__(self, other):
        return cmp(self.value, other.value)

    def __coerce__(self, other):
        return (self, self.__class__(other))

    def __repr__(self):
        return '%s: %s' % (self.__class__.__name__, str(self))

    def unicode(self):
        return unicode(str(self), 'UTF-8')

    def __str__(self):
        pass

    def __int__(self):
        return int(self.value)

    def __long__(self):
        return long(self.value)

# int
class XSType_nofacet_int(XSType_nofacet):
    def _set_value_helper(self, value):
        try:
            return int(value)
        except:
            return self.get_default_value(plain=1)
    
    def __str__(self):
        return '%d' % self.value

    def _get_default_value_nofacet(self):
        return 0

    def __int__(self):
        return self.value

# short
class XSType_nofacet_short(XSType_nofacet_int):
    pass

# date
class XSType_nofacet_date(XSType_nofacet):
    def _set_value_helper(self, value):
        tmp = None
        # DateTime does not properly handle None
        if value is not None:
            if isinstance(value, DateTime.DateTime):
                # DateTime is unable to init from its own kind. ts, ts
                tmp = DateTime.DateTime(value.timeTime())
            else:
                try:
                    tmp = DateTime.DateTime(value)
                except:
                    tmp = self.get_default_value(plain=1)
        else:
            tmp = self.get_default_value(plain=1)
        return tmp.earliestTime()
        
    def __str__(self):
        return self.value.strftime('%Y-%m-%d')

    def _get_default_value_nofacet(self):
        return DateTime.DateTime()

    def __int__(self):
        return int(self.value.timeTime())

    def __long__(self):
        return long(self.value.timeTime())

# dateTime
class XSType_nofacet_dateTime(XSType_nofacet):
    def _set_value_helper(self, value):
        tmp = None
        if value is not None:
            if isinstance(value, DateTime.DateTime):
                # DateTime is unable to init from its own kind. ts, ts
                tmp = DateTime.DateTime(value.timeTime())
            else:
                try:
                    tmp = DateTime.DateTime(value)
                except:
                    tmp = self.get_default_value(plain=1)
        else:
            tmp = self.get_default_value(plain=1)
        return tmp
            
    def __str__(self):
        return self.value.strftime('%Y-%m-%dT%H:%M:%S')

    def _get_default_value_nofacet(self):
        return DateTime.DateTime()

    def __int__(self):
        return int(self.value.timeTime())

    def __long__(self):
        return long(self.value.timeTime())

# boolean
class XSType_nofacet_boolean(XSType_nofacet):
    def _set_value_helper(self, value):
        if value in ('true', '1', 1):
           return 1 
        elif value in ('false', '0', 0, None):
            return 0
        elif value:
            return 1
        else:
            return self.get_default_value(plain=1)
            
    def __str__(self):
        if self.value == 1:
            return 'true'
        elif self.value == 0:
            return 'false'
        return 'false'

    def _get_default_value_nofacet(self):
        return 0

# string
class XSType_nofacet_string(XSType_nofacet):
    def _set_value_helper(self, value):
        if isinstance(value, type('')):
            return unicode(value, 'UTF-8')
        elif isinstance(value, type(u'')):
            return value
        else:
            return self.get_default_value(plain=1)
            
    def __str__(self):
        return self.value.encode('UTF-8')

    def _get_default_value_nofacet(self):
        return 'string'

# float
class XSType_nofacet_float(XSType_nofacet):
    def _set_value_helper(self, value):
        try:
            return float(value)
        except:
            return self.get_default_value(plain=1)
    def __str__(self):
        return '%g' % self.value

    def _get_default_value_nofacet(self):
        return float(0)

# double
class XSType_nofacet_double(XSType_nofacet_float):
    pass

# xsd:integer -> python:long
class XSType_nofacet_integer(XSType_nofacet):
    def _set_value_helper(self, value):
        if value is not None:
            try:
                return long(value)
            except (TypeError, ValueError):
                # imitate Formulators behaviour
                raise XSValidationError, 'not_integer: %s "%s"' % (type(value), value)
        else:
            return self.get_default_value(plain=1)
            
    def __str__(self):
        return '%d' % self.value

    def _get_default_value_nofacet(self):
        return long(0)

# to make some of the facet work
class XSType_nofacet_positiveInteger(XSType_nofacet_integer):
    def _set_value_helper(self, value):
        try:
            return long(value)
        except:
            return self.get_default_value(plain=1)

    def _get_default_value_nofacet(self):
        return long(1) # positive!

class XSType_nofacet_nonNegativeInteger(XSType_nofacet_integer):
    def _get_default_value_nofacet(self):
        return long(0)

class XSType_nofacet_nonPositiveInteger(XSType_nofacet_integer):
    def _get_default_value_nofacet(self):
        return long(0)

class XSType_nofacet_negativeInteger(XSType_nofacet_integer):
    def _get_default_value_nofacet(self):
        return long(-1)

class XSType_nofacet_long(XSType_nofacet_integer):
    def _get_default_value_nofacet(self):
        return long(0)

class XSType_nofacet_byte(XSType_nofacet_int):
    def _get_default_value_nofacet(self):
        return int(0)

class XSType_nofacet_unsignedLong(XSType_nofacet_integer):
    def _get_default_value_nofacet(self):
        return long(0)

class XSType_nofacet_unsignedInt(XSType_nofacet_integer):
    # unsignedInt is too large for Python int()
    def _get_default_value_nofacet(self):
        return long(0)

class XSType_nofacet_unsignedShort(XSType_nofacet_int):
    def _get_default_value_nofacet(self):
        return int(0)

class XSType_nofacet_unsignedByte(XSType_nofacet_int):
    def _get_default_value_nofacet(self):
        return int(0)

registry = {
    'string': XSType_nofacet_string,
    'int': XSType_nofacet_int,
    'short': XSType_nofacet_short,
    'date': XSType_nofacet_date,
    'dateTime': XSType_nofacet_dateTime,
    'boolean': XSType_nofacet_boolean,
    'float': XSType_nofacet_float,
    'double': XSType_nofacet_double,
    'integer': XSType_nofacet_integer,
    'positiveInteger': XSType_nofacet_positiveInteger,
    'nonNegativeInteger': XSType_nofacet_nonNegativeInteger,
    'nonPositiveInteger': XSType_nofacet_nonPositiveInteger,
    'negativeInteger':XSType_nofacet_negativeInteger,
    'long': XSType_nofacet_long,
    'byte': XSType_nofacet_byte,
    'unsignedLong': XSType_nofacet_unsignedLong,
    'unsignedInt': XSType_nofacet_unsignedInt,
    'unsignedShort': XSType_nofacet_unsignedShort,
    'unsignedByte': XSType_nofacet_unsignedByte
    }
