# $Header: /cvsroot/SchemaEditor/XSWebType.py,v 1.11 2003/08/26 17:24:31 spuelrich Exp $
import XSType
import XSType_nofacet
import XSField
from XSTypeRegistry import globalXSTypeRegistry
from AccessControl import ClassSecurityInfo
from Persistence import Persistent
from xsdTools import getElementValue

class XSWebType(Persistent):
    """mix-in class for all TTW XSTypes"""
    security = ClassSecurityInfo()
    nodepath = None

    # allow access to functions from XSType_nofacet and XSType
    security.declarePublic('unicode')
    security.declarePublic('get_nofacet_value')
    security.declarePublic('set_value')

    security.declarePublic('render')
    def render(self, context):
        """Renders the given or the current value as string"""
        xsd_node = context.REQUEST.node
        return context.service_editor.getWidget(xsd_node).render_real(self)

    security.declarePublic('render_field')
    def render_field(self, xsi_node, value=None):
        """Renders the types field with the current value."""
        # don't validate the value from the XSI. it might be invalid!
        if self.facets.has_key('enumeration'):
            return self.facets['enumeration'].render_field(xsi_node, value)
        else:
            if value is None:
                value = getElementValue(xsi_node)
            nf = self.nofacet_class(value)
            return self.field.render(xsi_node, nf.get_value())

    security.declarePublic('create_instance')
    def create_instance(self, context, value=None):
        """Creates a new TEXT_NODE under context.REQUEST.xsi_parent_node,
        which contains a string representation of the current value."""
        xsi_parent = context.REQUEST.xsi_parent_node
        if value is None:
            # compute default value as a XSType_nofacet
            value = self.get_default_value()
        node = xsi_parent.ownerDocument.createTextNode(value.unicode())
        xsi_parent.appendChild(node)
        return 'created %s instance' % (self.name)

    security.declarePublic('validate_from_request')
    def validate_from_request(self, xsi_node, REQUEST):
        """Grabs the new value from REQUEST (via field) and validates it."""
        value = None
        # circumvent my own validate_from_request, if there is a xsd:enumeration facet
        if self.facets.has_key('enumeration'):
            value = self.facets['enumeration'].validate_field(xsi_node, REQUEST)
        else:
            value = self.field.validate(xsi_node, REQUEST)
        return value

    security.declarePublic('has_simple_content')
    def has_simple_content(self):
        return 1

    security.declarePublic('get_info')
    def get_info(self):
        return self.__doc__

# ugly, but true.
# the field instances have to be created here, because this is the only way to initialize them correctly.
# don't know why.

class XSWebType_int(XSWebType, XSType.XSType_int):
    """
    xsd:int
    maxInclusive = 2147483647
    minInclusive = -2147483648"""
    field = XSField.XSField_int(XSField.XSField_int.meta_type)
    facet_class = XSType.XSType_int

class XSWebType_short(XSWebType, XSType.XSType_short):
    """
    xsd:short
    maxInclusive = 32767
    minInclusive = -32768"""
    field = XSField.XSField_short(XSField.XSField_short.meta_type)
    facet_class = XSType.XSType_short

class XSWebType_date(XSWebType, XSType.XSType_date):
    """
    xsd:date"""
    field = XSField.XSField_date(XSField.XSField_date.meta_type, date_separator='-', date_only=1)
    facet_class = XSType.XSType_date

class XSWebType_dateTime(XSWebType, XSType.XSType_dateTime):
    """
    xsd:dateTime"""
    field = XSField.XSField_dateTime(XSField.XSField_dateTime.meta_type, date_separator='-')
    facet_class = XSType.XSType_dateTime

class XSWebType_string(XSWebType, XSType.XSType_string):
    """
    xsd:string"""
    field = XSField.XSField_string(XSField.XSField_string.meta_type, whitespace_preserve="1", unicode="1")
    facet_class = XSType.XSType_string

class XSWebType_float(XSWebType, XSType.XSType_float):
    """
    xsd:float"""
    field = XSField.XSField_float(XSField.XSField_float.meta_type)
    facet_class = XSType.XSType_float

class XSWebType_double(XSWebType, XSType.XSType_double):
    """
    xsd:double"""
    field = XSField.XSField_double(XSField.XSField_double.meta_type)
    facet_class = XSType.XSType_double
    
class XSWebType_boolean(XSWebType, XSType.XSType_boolean):
    """
    xsd:boolean"""
    field = XSField.XSField_boolean(XSField.XSField_boolean.meta_type)
    facet_class = XSType.XSType_boolean
    
class XSWebType_integer(XSWebType, XSType.XSType_integer):
    """
    xsd:integer
    fractionDigits = 0
    any integer number"""
    field = XSField.XSField_integer(XSField.XSField_integer.meta_type)
    facet_class = XSType.XSType_integer
    
class XSWebType_positiveInteger(XSWebType, XSType.XSType_positiveInteger):
    """
    xsd:positiveInteger
    minInclusive = 1"""
    field = XSField.XSField_positiveInteger(XSField.XSField_positiveInteger.meta_type)
    facet_class = XSType.XSType_positiveInteger
    
class XSWebType_nonNegativeInteger(XSWebType, XSType.XSType_nonNegativeInteger):
    """
    xsd:nonNegativeInteger
    minInclusive = 0"""
    field = XSField.XSField_nonNegativeInteger(XSField.XSField_nonNegativeInteger.meta_type)
    facet_class = XSType.XSType_nonNegativeInteger

class XSWebType_nonPositiveInteger(XSWebType, XSType.XSType_nonPositiveInteger):
    """
    xsd:nonPositiveInteger
    maxInclusive = 0"""
    field = XSField.XSField_nonPositiveInteger(XSField.XSField_nonPositiveInteger.meta_type)
    facet_class = XSType.XSType_nonPositiveInteger

class XSWebType_negativeInteger(XSWebType, XSType.XSType_negativeInteger):
    """
    xsd:negativeInteger
    maxInclusive = -1"""
    field = XSField.XSField_negativeInteger(XSField.XSField_negativeInteger.meta_type)
    facet_class = XSType.XSType_negativeInteger

class XSWebType_long(XSWebType, XSType.XSType_long):
    """
    xsd:long
    maxInclusive = 9223372036854775807
    minInclusive = -9223372036854775808"""
    field = XSField.XSField_long(XSField.XSField_long.meta_type)
    facet_class = XSType.XSType_long

class XSWebType_byte(XSWebType, XSType.XSType_byte):
    """
    xsd:byte
    maxInclusive = 127
    minInclusive = -128"""
    field = XSField.XSField_byte(XSField.XSField_byte.meta_type)
    facet_class = XSType.XSType_byte

class XSWebType_unsignedLong(XSWebType, XSType.XSType_unsignedLong):
    """
    xsd:unsignedLong
    maxInclusive = 18446744073709551615
    minInclusive = 0"""
    field = XSField.XSField_unsignedLong(XSField.XSField_unsignedLong.meta_type)
    facet_class = XSType.XSType_unsignedLong

class XSWebType_unsignedInt(XSWebType, XSType.XSType_unsignedInt):
    """
    xsd:unsignedInt
    maxInclusive = 4294967295
    minInclusive = 0"""
    field = XSField.XSField_unsignedInt(XSField.XSField_unsignedInt.meta_type)
    facet_class = XSType.XSType_unsignedInt

class XSWebType_unsignedShort(XSWebType, XSType.XSType_unsignedShort):
    """
    xsd:unsignedShort
    maxInclusive = 65535
    minInclusive = 0"""
    field = XSField.XSField_unsignedShort(XSField.XSField_unsignedShort.meta_type)
    facet_class = XSType.XSType_unsignedShort

class XSWebType_unsignedByte(XSWebType, XSType.XSType_unsignedByte):
    """
    xsd:unsignedByte
    maxInclusive = 255
    minInclusive = 0"""
    field = XSField.XSField_unsignedByte(XSField.XSField_unsignedByte.meta_type)
    facet_class = XSType.XSType_unsignedByte

def initialize_global_type_registry():
    globalXSTypeRegistry.register_type(XSWebType_int())
    globalXSTypeRegistry.register_type(XSWebType_short())
    globalXSTypeRegistry.register_type(XSWebType_double())
    globalXSTypeRegistry.register_type(XSWebType_float())
    globalXSTypeRegistry.register_type(XSWebType_string())
    globalXSTypeRegistry.register_type(XSWebType_dateTime())
    globalXSTypeRegistry.register_type(XSWebType_date())
    globalXSTypeRegistry.register_type(XSWebType_boolean())
    globalXSTypeRegistry.register_type(XSWebType_integer())
    globalXSTypeRegistry.register_type(XSWebType_positiveInteger())
    globalXSTypeRegistry.register_type(XSWebType_nonNegativeInteger())
    globalXSTypeRegistry.register_type(XSWebType_nonPositiveInteger())
    globalXSTypeRegistry.register_type(XSWebType_negativeInteger())
    globalXSTypeRegistry.register_type(XSWebType_long())
    globalXSTypeRegistry.register_type(XSWebType_byte())
    globalXSTypeRegistry.register_type(XSWebType_unsignedLong())
    globalXSTypeRegistry.register_type(XSWebType_unsignedInt())
    globalXSTypeRegistry.register_type(XSWebType_unsignedShort())
    globalXSTypeRegistry.register_type(XSWebType_unsignedByte())
