"""Hilfsfunktionen

sollte auch in PythonScripts verwendbar sein

$Header: /cvsroot/SchemaEditor/xsdTools.py,v 1.11 2003/08/30 09:18:27 spuelrich Exp $
"""

__version__ = "$Revision: 1.11 $"[11:-2]

from AccessControl import ModuleSecurityInfo, getSecurityManager
from XSExceptions import XSUndeclaredElement
security = ModuleSecurityInfo()

security.declarePublic(
    'xsdURI',
    'seURI',
    'get_document_encoding',
    'setElementValue',
    'getElementValue',
    'getChildNodesByTagNameNS',
    'getFirstElementChild',
    'getNextElementSibling',
    'getFirstElementSibling',
    'minimize_paths',
    # make thisexception available to PythonScripts
    'XSUndeclaredElement'
    )

xsdURI = "http://www.w3.org/2001/XMLSchema"
seURI = "http://zope.fh-augsburg.de/Diplomarbeiten/SchemaEditor/namespace"

def get_document_encoding(node):
    "Return the encoding of the object's document."
    if (node.getDOMObj().nodeType == node.DOCUMENT_NODE):
        encoding = node.getDOMObj().encoding
    else:
        encoding = node.getDOMObj().ownerDocument.encoding
    return encoding

def setElementValue(node, data):
    """sets the value of the first TEXT_NODE child.
    If it doesn't exist, it will be created."""

    # encoding of data has to be done by type.to_unicode()!!
    # XXX replace all TEXT_NODE siblings with one new containing the new value
    if node.hasChildNodes():
        tn = node.firstChild
        if tn.nodeType == tn.TEXT_NODE:
            tn.replaceData(0, tn.length, data)
        else:
            newnode = node.ownerDocument.createTextNode(data)
            node.insertBefore(newnode, tn)
    else:
        newnode = node.ownerDocument.createTextNode(data)
        node.appendChild(newnode)

def getElementValue(node):
    """returns the value of the first TEXT_NODE child."""
    # XXX collect the values of all TEXT_NODE siblings
    tn = node.firstChild
    if tn is None or tn.nodeType != tn.TEXT_NODE:
        return ''
    else:
        content = tn.data
        return content

def getChildNodesByTagNameNS(node, localname, uri=xsdURI):
    nl=[]
    for child in node.childNodes:
        if(child.namespaceURI == uri and
           child.localName == localname):
            nl.append(child)
    return nl

def getFirstElementChild(node):
    """returns the first child of node, which is an ELEMENT_NODE"""
    child = node.getDOMObj().firstChild
    while child is not None:
        if child.nodeType == child.ELEMENT_NODE:
            return node.wrapDOMObj(child)
        child = child.nextSibling
    return None

def getNextElementSibling(node):
    """returns the next sibling, which is of type ELEMENT_NODE or None"""
    sibling = node.getDOMObj().nextSibling
    while sibling is not None:
        if sibling.nodeType == sibling.ELEMENT_NODE:
            return node.wrapDOMObj(sibling)
        sibling = sibling.nextSibling
    return None

def getFirstElementSibling(node):
    """returns the first of all siblings, which is a element or the node self."""
    if node is None:
        return None
    else:
        return getFirstElementChild(node.parentNode)

def hasAncestorWithNameNS(node, localname, uri=xsdURI):
    """tests whether the node has an ancestor, whose localName == localname and namespaceURI == uri"""
    node = node.parentNode
    while (node is not None and node.nodeType == node.ELEMENT_NODE):
        if (node.localName == localname and node.namespaceURI == uri):
            return 1
        node = node.parentNode
    return 0

def minimize_paths(path, *more_paths):
    """concatenats all paths and reduces the length of the result, by eliminating redundant elements"""
    # /fuu/bar/baz/foo/bar/buz -> /foo/bar/baz/buz
    for ext_path in more_paths:
        counter = 0
        while counter < len(ext_path):
            if path[counter] != ext_path[counter]:
                break
            counter += 1
        path = path + ext_path[counter:]
    return path

security.apply(globals())
